## Utility to Calculate PPV, Sensitivity and MCC of the results.
import sys
from math import sqrt

if len(sys.argv) < 5:
	print("Use in this format: python CalculateAccuracy.py {Genome} {Rna Pos} {Results} {Shuffled: True/False}")
	sys.exit(0)
	
rnaposFile = open(sys.argv[2],'r')
resultFile = open(sys.argv[3],'r')

rnaPositions = []
resPositions = []

bShuffled = sys.argv[4]

## get the positions of all known ncRNA
rnaposText = rnaposFile.read()
rnaposText = rnaposText.split("\n")
for info in rnaposText:
	if info == '':
		break
	rnaPositions.append(info.split("\t"))
rnaposFile.close()

detectedKnownRna = [0] * len(rnaPositions) ## list to remember if we have detected a specific known ncRNA

## get the positions of the detected ncRNA
resultText = resultFile.read()
resultText = resultText.split("\n")
for info in resultText:
	if info == '':
		break
	resPositions.append(info.split("\t"))
		
## calculate PPV and Sensitivity
TP = 0
FN = 0
FP = 0
NumMCC= 0
for res in resPositions:
	indivTP = 0
	indivFN = 0
	indivFP = 0
	foundIntersect = False
	x = int(res[0])
	y = int(res[1])
	count = 0
	for rna in rnaPositions:
		## get minimum positions and maximum position and list size will be max - min
		nX = int(rna[0])
		nY = int(rna[1])
		if ((x <= nX and y >= nY) or (x >= nX and x <= nY) or (y >= nX and y <= nY)):			
			foundIntersect = True
			detectedKnownRna[count] = 1
			
			## found intersection - now determine overlap
			if x <= nX and y >= nY:
				indivTP += nY - nX
				indivFP += (y - nY) + (nX - x)
			elif x >= nX and x <= nY and y > nY:
				indivTP += nY - x + 1
				indivFP += y - nY
				indivFN += x - nX
			elif x >= nX and y <= nY:
				indivTP += y - x + 1
				indivFN += nY - y
			elif x < nX and y >= nX and y <= nY:
				indivTP += y - nX + 1
				indivFP += nX - x
				indivFN += nY - y
		count += 1
		
	TP += indivTP
	FN += indivFN
	FP += indivFP

	indivSens = -1 ## -1 means it has no meaning as it has no intersection with a known ncRNA
	indivPPV = -1
	indivMCC = -1
	## calculate variables
	if foundIntersect:
		indivSens = float(indivTP)/(indivTP + indivFN)
		indivPPV = float(indivTP)/(indivTP + indivFP)
		indivMCC = sqrt(indivPPV * indivSens)
	elif bShuffled == "True":
		FP += y - x
	if indivMCC >= 0.5:
		NumMCC += 1
	print(res[0] + "\t" + res[1] + "\tSensitivity: "+ str(indivSens) + "\tPPV: " + str(indivPPV) + "\tMCC: " + str(indivMCC) + ("*" if indivMCC >= 0.5 else "")) 

Sens = float(TP)/(TP + FN)
PPV = float(TP)/(TP + FP)
MCC = sqrt(Sens *PPV)
FDR = float(FP)/(TP+FP)
print("Detections with MCC > 0.5: " + str(NumMCC))
print("FDR: " + str(FDR))
print("Total Sensitivity: "+ str(Sens) + "\tTotal PPV: " + str(PPV) + "\tTotal MCC: " + str(MCC)) 
print("TOTAL KNOWN NCRNA DETECTED: " + str(sum(detectedKnownRna)) + "/" + str(len(rnaPositions)) + " " + str(float(sum(detectedKnownRna))/len(rnaPositions)) + "%")
print("Percent of Result is Known: " + str(sum(detectedKnownRna)) + "/" + str(len(resPositions)) + " " + str(float(sum(detectedKnownRna))/len(resPositions)) + "%")